'use strict';

var fs = require('fs');
var path = require('path');
var markmapCommon = require('markmap-common');
var zod = require('zod');
var markmapLib = require('markmap-lib');
var zodValidationError = require('zod-validation-error');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var fs__default = /*#__PURE__*/_interopDefault(fs);
var path__default = /*#__PURE__*/_interopDefault(path);

// src/index.ts

// src/template/markmap-style.ts
var markmap_style_default = (darkThemeCssSelector) => (
  /* css */
  `
  .markmap-wrap {
    position: relative;
    width: 100%;
    height: 100%;
    display: flex;
    justify-content: center;
  }

  ${darkThemeCssSelector} .markmap-wrap {
    background: #1a1a1a;
  }

  .markmap-wrap.fullscreen {
    background: #ffffff;
  }

  .markmap-wrap.fullscreen svg {
    height: auto !important;
  }

  ${darkThemeCssSelector} .markmap-wrap.fullscreen {
    background: #1a1a1a;
  }
      
  .markmap-wrap>svg {
    display: block;
    width: 100%;
    max-height: 100%;
    height: auto;
    min-height: 150px;
    margin: 0;
    padding: 0;
  }
  
  .mm-toolbar {
    position: absolute;
    right: .5em;
    bottom: .5em;
  }
  
  ${darkThemeCssSelector} .markmap {
    --markmap-code-bg: #1a1b26;
    --markmap-code-color: #ddd;
    --markmap-circle-open-bg: #444;
    --markmap-text-color: #eee;
  }
  
  ${darkThemeCssSelector} .mm-toolbar {
    --un-border-opacity: 1;
    border-color: rgb(82 82 91 / var(--un-border-opacity));
    --un-bg-opacity: 1;
    background-color: rgb(39 39 42 / var(--un-bg-opacity));
    --un-text-opacity: 1;
    color: rgb(161 161 170 / var(--un-text-opacity));
  }
  `
);

// src/template/markmap-wrapper.ts
var markmap_wrapper_default = (root, jsonOptions, id, height) => (
  /* html */
  `
    <div class="markmap-wrap" 
      ${id ? `id="${id}"` : ""}
      ${height ? `style="height: ${height}"` : ""}
    >
      <script type="application/json">${root}</script>
      <script type="application/json">${jsonOptions}</script>
    </div>
    `
);

// src/utils/helper.ts
var ExtendedMap = class extends Map {
  entry(key) {
    const entryContext = {
      and: (modifier) => {
        if (this.has(key)) modifier(this.get(key));
        return entryContext;
      },
      or_insert: (defaultFactory) => {
        if (!this.has(key)) this.set(key, defaultFactory());
        return this.get(key);
      }
    };
    return entryContext;
  }
};
var AVAILABLE_PROVIDERS = ["jsdelivr", "fastly", "unpkg", "custom"];
var DEFAULT_PROVIDER_FACTORIES = {
  jsdelivr: (path2) => `https://cdn.jsdelivr.net/npm/${path2}`,
  fastly: (path2) => `https://fastly.jsdelivr.net/npm/${path2}`,
  unpkg: (path2) => `https://unpkg.com/${path2}`,
  custom: (path2) => `https://fastly.jsdelivr.net/npm/${path2}`
};
var DEFAULT_PROVIDER_NAME = "fastly";
var createUrlBuilder = (defaultProviderName = DEFAULT_PROVIDER_NAME) => {
  const urlBuilder2 = new markmapCommon.UrlBuilder();
  for (const key of AVAILABLE_PROVIDERS) {
    urlBuilder2.setProvider(key, DEFAULT_PROVIDER_FACTORIES[key]);
  }
  urlBuilder2.provider = defaultProviderName;
  return urlBuilder2;
};
var getTransformer = (defaultProvider) => {
  const urlBuilder2 = createUrlBuilder(defaultProvider);
  const transformer2 = new markmapLib.Transformer();
  transformer2.urlBuilder = urlBuilder2;
  return transformer2;
};
var configSchema = zod.z.object({
  globalOptions: zod.z.object({}).passthrough().optional().default({}),
  CDN: zod.z.enum(AVAILABLE_PROVIDERS).optional().default(DEFAULT_PROVIDER_NAME),
  customCDN: zod.z.string().url().optional(),
  darkThemeCssSelector: zod.z.string().optional().default(".dark")
});
var checkOldConfig = (config = {}) => {
  const deprecatedKeys = [
    "pjax",
    "katex",
    "prism",
    "userCDN",
    "lockView",
    "fixSVGAttrNaN"
  ];
  const hasDeprecatedKeys = deprecatedKeys.some((key) => key in config);
  if (!hasDeprecatedKeys) return config;
  const WARN_PREFIX = "\x1B[33mWARN\x1B[0m [hexo-markmap]";
  console.warn(`${WARN_PREFIX} Deprecated configuration options detected. These options will be ignored:`, deprecatedKeys);
  if ("userCDN" in config) {
    console.warn(`${WARN_PREFIX} The userCDN option is deprecated. Please set the CDN option to 'custom' and use the \`customCDN\` option instead.`);
  }
  if ("lockView" in config) {
    console.warn(`${WARN_PREFIX} The lockView option is deprecated and will be ignored.`);
  }
  return Object.fromEntries(
    Object.entries(config).filter(([key]) => !deprecatedKeys.includes(key))
  );
};
var parseConfig = (config = {}) => {
  const filteredConfig = checkOldConfig(config);
  const parsedConfig = configSchema.safeParse(filteredConfig);
  if (!parsedConfig.success) {
    const validationError = zodValidationError.fromError(parsedConfig.error);
    throw new Error(validationError.message);
  }
  return parsedConfig.data;
};
var frontmatterSchema = zod.z.object({
  id: zod.z.string().optional(),
  options: zod.z.object({}).passthrough().optional().default({}),
  markmap: zod.z.object({}).passthrough().optional().default({})
});
var parseFrontmatter = (data = {}, identifier) => {
  const parsedData = frontmatterSchema.safeParse(data);
  if (!parsedData.success) {
    const validationError = zodValidationError.fromError(parsedData.error);
    throw new Error(validationError.message);
  }
  return parsedData.data;
};

// src/index.ts
var userConfig = parseConfig(hexo.config["hexo_markmap"]);
var transformer = getTransformer();
var { urlBuilder } = transformer;
urlBuilder.provider = userConfig.CDN;
if (userConfig.customCDN) {
  urlBuilder.setProvider("custom", (path2) => `${userConfig.customCDN}${path2}`);
}
var js = hexo.extend.helper.get("js").bind(hexo);
var css = hexo.extend.helper.get("css").bind(hexo);
var pageAssetsMap = new ExtendedMap();
hexo.extend.tag.register("markmap", function([height], content) {
  const { root, features, frontmatter: rawFrontmatter } = transformer.transform(content);
  const frontmatter = parseFrontmatter(rawFrontmatter);
  const { id, markmap, options } = frontmatter;
  const mergedOptions = {
    ...userConfig.globalOptions,
    ...options,
    ...markmap
    // original property name defined in markmap-lib
  };
  const { styles = [], scripts = [] } = transformer.getUsedAssets(features);
  const wrapHTML = markmap_wrapper_default(JSON.stringify(root), JSON.stringify(mergedOptions), id, height);
  const assetsHTML = [
    ...markmapCommon.persistCSS(styles),
    ...markmapCommon.persistJS(scripts, {
      getMarkmap: () => window.markmap,
      root
    })
  ];
  const { slug } = this;
  slug && pageAssetsMap.entry(slug).and((set) => assetsHTML.forEach(set.add, set)).or_insert(() => new Set(assetsHTML));
  return wrapHTML.trim();
}, { ends: true });
hexo.extend.generator.register("markmap_asset", () => [{
  path: "js/markmap.js",
  data: () => fs__default.default.createReadStream(path__default.default.resolve(__dirname, "./markmap-init.global.js"))
}, {
  path: "css/markmap.css",
  data: () => markmap_style_default(userConfig.darkThemeCssSelector)
}]);
hexo.extend.filter.register("after_post_render", function(data) {
  const { slug } = data;
  const pageAssets = pageAssetsMap.get(slug) ?? [];
  const VIEW_VERSION = "0.18.10";
  const TOOLBAR_VERSION = "0.18.10";
  const basePackages = [
    "d3@7",
    `markmap-view@${VIEW_VERSION}`,
    `markmap-toolbar@${TOOLBAR_VERSION}`
  ];
  const basePackageScripts = basePackages.map(
    (name) => `<script src="${urlBuilder.getFullUrl(name)}"></script>`
  );
  const basePackageStyles = [
    urlBuilder.getFullUrl(
      `markmap-toolbar@${TOOLBAR_VERSION}/dist/style.css`
    )
  ].map((path2) => `<link rel="stylesheet" href="${path2}">`);
  const combinedAssetHTML = [
    ...basePackageStyles,
    ...basePackageScripts,
    ...pageAssets,
    css("/css/markmap.css"),
    js("/js/markmap.js")
  ];
  data.content += combinedAssetHTML.join("");
});
